/*********************************************************************
*                                                                    *          
*                              FLY                                   *
*                           --E#Y#E--				     *
*                         ===MUSCLE===                               *
*                                                                    *
*********************************************************************/

/* Header Files */
#include "opengl.h"

/* Define Key */
#define XK_space                         0x0020
#define XK_Escape                        0xff1b
#define XK_F1                            0xffbe
#define XK_F2                            0xffbf
#define XK_F3				 0xffc0
#define XK_F4				 0xffc1
#define XK_Left                          0xff51  
#define XK_Up                            0xff52  
#define XK_Right                         0xff53  
#define XK_Down                          0xff54  
#define XK_a                             0x0061
#define XK_b                             0x0062
#define XK_d                             0x0064
#define XK_e                             0x0065
#define XK_q                             0x0071 
#define XK_s                             0x0073 
#define XK_w                             0x0077
#define XK_n                             0x006e
#define XK_m                             0x006d

/* Define Pi */
#define Pi 3.14159

/*-------------------------------------------------------------------------*/

GLfloat roll;
GLfloat yaw;
GLfloat pitch;
GLfloat planeX;
GLfloat planeY;
GLfloat planeZ;
GLfloat rootY;
GLfloat viewZ;

/* RGB Colour */
float VA0;
float VA1;    
float VA5;
float VA10;

/* Load Textures */
FILE *rockbmp;
FILE *pixel;
/* 4 Landscapes Files */
FILE *landsc1;
FILE *landsc2;
FILE *landsc3;
FILE *landsc4;
FILE *setting;

/*---------------------------------------------------------------------------*/

/* Init Triangles */
void init(char night)
{	
	/* Background Color */
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	
	if(night)
	{
		glClearColor( 0.1f , 0.1f , 0.1f , 1.0f );
	}
	else
	{
		glClearColor( 0.3f , 0.4f , 0.7f , 1.0f );
	}
}

/*---------------------------------------------------------------------------*/

/* The FLY Texture */
void texture()
{
	/* Texture Shader */
	CreateShader();
	CreateProgram();
	Location(0, "a_Vertex");
	Location(1, "a_TexCoord");

	/* Load Textures */
	//rockbmp=fopen("PIXEL/rock.bmp");

	/* Enable Texture */
	glEnable(GL_TEXTURE_2D);
	/* Textures Object */
	GLuint fRock = 0;
	glGenTextures(1, &fRock);
	/* Bind Textures */
	glActiveTexture(GL_TEXTURE0);
	glBindTexture(GL_TEXTURE_2D, fRock);
	
	/* Filtering Mode */
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	/* Image Data */
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, 256, 256, 0, GL_BGR,
			GL_UNSIGNED_BYTE, rockbmp);
	
	glEnableVertexAttribArray(0); 
    	glEnableVertexAttribArray(1); 
}

/*---------------------------------------------------------------------------*/

/* Render Landscape */
void Landscape(char colour)
{
	GLfloat projection[16];
	GLfloat modelview[16];

	/* Render Triangles */
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();
	glTranslatef(0.0f, 0.0f, 0.0f);

	/* Camera Look */
	gluLookAt(0.0f, 0.0f,  25.0f, 
		  0.0f, 0.0f, -25.0f, 
		  0.0f, 5.0f,  0.0f);

	/* Passing Data */
	glGetFloatv(GL_PROJECTION_MATRIX, projection);
	glGetFloatv(GL_MODELVIEW_MATRIX, modelview);
	//Uniform("texture0", 0);
	
	//glVertexAttribPointer((GLint)0, 3, GL_FLOAT, GL_FALSE, 0, 0);
	//glVertexAttribPointer((GLint)1, 2, GL_FLOAT, GL_FALSE, 0, 0);

	/* Rotation */
	glRotatef(-roll, 0.0f, 0.0f, 0.4f);
	glRotatef(-yaw, 0.0f, 0.4f, 0.0f);
	glRotatef(-pitch, 0.4f, 0.0f, 0.0f);
	glTranslatef(-planeX, -planeY, -planeZ);
	glLineWidth(1);		

	/* Settings */
	if(colour)
	{
		setting=fopen("DATA/setting1.fs","r");
		glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
	}
	else
	{
		setting=fopen("DATA/setting2.fs","r");
		glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
	}
	
	/* Open Landscape File */
	pixel=fopen("DATA/pixel.fs","r");
	landsc1=fopen("DATA/landsc1.fs","r");
	landsc2=fopen("DATA/landsc2.fs","r");
	landsc3=fopen("DATA/landsc3.fs","r");
	landsc4=fopen("DATA/landsc4.fs","r");

	/* Begin Triangles */
	glBegin(GL_TRIANGLES);

	//glTexCoord2f(0.0f, 1.0f);glVertex3f(-0.5f,-0.5f,-2.0f);
	//glTexCoord2f(0.0f, 0.0f);glVertex3f( 0.5f,-0.5f,-2.0f);
	//glTexCoord2f(1.0f, 0.0f);glVertex3f( 0.0f, 0.5f,-2.0f);

	/* Render Texture */	
	while((fscanf(pixel,"%f %f", &gl.tex.s, &gl.tex.t )) != EOF)
	{ 
		glTexCoord2f((gl.tex.s-30), (gl.tex.t-30));
	}
	fscanf(setting,"%s %f %f %f ",&ti, &gl.col.r, &gl.col.g, &gl.col.b);
	glColor3f(gl.col.r, gl.col.g, gl.col.b);	
	while((fscanf(landsc1,"%f %f %f", &gl.tri.x, &gl.tri.y, &gl.tri.z))!=EOF)
	{ 
		//printf("%02ff , %02ff , %02ff\n", x, y, z); 
		glVertex3f((gl.tri.x-30), gl.tri.y, (gl.tri.z-30));
	}	
	fscanf(setting,"%s %f %f %f ",&ti, &gl.col.r, &gl.col.g, &gl.col.b);
	glColor3f(gl.col.r, gl.col.g, gl.col.b);	
	while((fscanf(landsc2,"%f %f %f", &gl.tri.x, &gl.tri.y, &gl.tri.z))!=EOF)
	{ 
		//printf("%02ff , %02ff , %02ff\n", x, y, z); 
		glVertex3f((gl.tri.x-30), gl.tri.y, (gl.tri.z+30));
	}
	fscanf(setting,"%s %f %f %f ",&ti, &gl.col.r, &gl.col.g, &gl.col.b);
	glColor3f(gl.col.r, gl.col.g, gl.col.b);	
	while((fscanf(landsc3,"%f %f %f", &gl.tri.x, &gl.tri.y, &gl.tri.z))!=EOF)
	{ 
		//printf("%02ff , %02ff , %02ff\n", x, y, z); 
		glVertex3f((gl.tri.x+30), gl.tri.y, (gl.tri.z-30));
	}
	fscanf(setting,"%s %f %f %f ",&ti, &gl.col.r, &gl.col.g, &gl.col.b);
	glColor3f(gl.col.r, gl.col.g, gl.col.b);	
	while((fscanf(landsc4,"%f %f %f", &gl.tri.x, &gl.tri.y, &gl.tri.z))!=EOF)
	{ 
		//printf("%02ff , %02ff , %02ff\n", x, y, z); 
		glVertex3f((gl.tri.x+30), gl.tri.y, (gl.tri.z+30));
	}
	/* End Triangles */
	glEnd();

	/* File Close */
	//DeleteShader();
	//fclose(rockbmp);
	fclose(pixel);

	/* File Close */	
	fclose(landsc1);
	fclose(landsc2);
	fclose(landsc3);
	fclose(landsc4);
	fclose(setting);
}

/*-------------------------------------------------------------------------*/

/* Render Cockpit */
void Cockpit()
{
	/* Rotation */
	glRotatef(roll, 0.0f, 0.0f ,0.1f);
	glRotatef(yaw, 0.0f, 0.1f ,0.0f);
	glRotatef(pitch, 0.1f, 0.0f ,0.0f);
	glTranslatef(planeX, planeY, planeZ + viewZ + 23.499f);
	glLineWidth(2);	

	/* Render Cockpit */
	glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
	/* Begin Quads */
	glBegin(GL_QUADS);
		/* Cockpit */
		glColor4f( 0.9f, 0.9f, 0.9f, 1.0f);
		glVertex3f(-0.3f, -0.2f, 0.3f );
		glVertex3f( 0.3f, -0.2f, 0.3f );
		glVertex3f( 0.3f,  0.3f, 0.3f );
		glVertex3f(-0.3f,  0.3f, 0.3f );
		
		glColor4f( 0.0 +VA10, 1.0 -VA5, 0.0 +VA0, 1.0 +VA0);		
		glVertex3f(-0.5f, -0.5f, 0.0f );
		glVertex3f( 0.5f, -0.5f, 0.0f );
		glVertex3f( 0.5f,  0.5f, 0.0f );
		glVertex3f(-0.5f,  0.5f, 0.0f );
		
		glVertex3f(-0.7f, -0.5f, 1.5f );
		glVertex3f( 0.7f, -0.5f, 1.5f );
		glVertex3f( 0.5f, -0.5f, 0.0f );
		glVertex3f(-0.5f, -0.5f, 0.0f );
		
		glVertex3f(-0.7f,  0.5f, 1.5f );
		glVertex3f( 0.7f,  0.5f, 1.5f );
		glVertex3f( 0.5f,  0.5f, 0.0f );
		glVertex3f(-0.5f,  0.5f, 0.0f );
		
		/* Terminal */
		glColor4f( 0.9f, 0.9f, 0.9f, 1.0f);
		glVertex3f(-1.0f, 1.09f + rootY, 0.5f -viewZ );
		glVertex3f( 1.0f, 1.09f + rootY, 0.5f -viewZ );
		glVertex3f( 1.0f, 1.05f + rootY, 0.5f -viewZ );
		glVertex3f(-1.0f, 1.05f + rootY, 0.5f -viewZ );
		
		glVertex3f(-1.0f, 0.64f + rootY, 0.5f -viewZ );
		glVertex3f( 1.0f, 0.64f + rootY, 0.5f -viewZ );
		glVertex3f( 1.0f, 0.60f + rootY, 0.5f -viewZ );
		glVertex3f(-1.0f, 0.60f + rootY, 0.5f -viewZ );
	/* End Quads */
	glEnd();
	
	/* Render Quads */
	glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
	/* Begin Quads */
	glBegin(GL_QUADS);
		/* Instruments */
		glColor4f(1.0f +VA0 ,1.0f -VA5 ,1.0f -VA10,0.4f -VA1);
		glVertex3f(-0.3f, -0.2f, 0.3f );
		glVertex3f( 0.3f, -0.2f, 0.3f );
		glVertex3f( 0.3f,  0.3f, 0.3f );
		glVertex3f(-0.3f,  0.3f, 0.3f );
		
		glColor4f( 0.2f, 0.2f, 0.2f, 1.0f);
		glVertex3f(-0.7f, -0.5f, 1.5f );
		glVertex3f( 0.7f, -0.5f, 1.5f );
		glVertex3f( 0.5f, -0.5f, 0.0f );
		glVertex3f(-0.5f, -0.5f, 0.0f );
		
		glVertex3f( 0.7f, 0.5f, 1.5f );
		glVertex3f(-0.7f, 0.5f, 1.5f );
		glVertex3f(-0.5f, 0.5f, 0.0f );
		glVertex3f( 0.5f, 0.5f, 0.0f );
		
		/* Terminal */
		glColor4f( 0.2f, 0.3f, 0.4f, 1.0f);
		glVertex3f(-1.0f, 0.60f + rootY, 0.5f -viewZ );
		glVertex3f( 1.0f, 0.60f + rootY, 0.5f -viewZ );
		glVertex3f( 1.0f, 0.64f + rootY, 0.5f -viewZ );
		glVertex3f(-1.0f, 0.64f + rootY, 0.5f -viewZ );
		
		glVertex3f(-1.0f, 1.09f + rootY, 0.5f -viewZ );
		glVertex3f( 1.0f, 1.09f + rootY, 0.5f -viewZ );
		glVertex3f( 1.0f, 1.05f + rootY, 0.5f -viewZ );
		glVertex3f(-1.0f, 1.05f + rootY, 0.5f -viewZ );
		
		glColor4f( 0.0f, 0.0f, 0.0f, 0.9f);
		glVertex3d(-1.0f, 0.6f + rootY, 0.5f -viewZ );
		glVertex3d( 1.0f, 0.6f + rootY, 0.5f -viewZ );
		glVertex3d( 1.0f, 1.1f + rootY, 0.5f -viewZ );
		glVertex3d(-1.0f, 1.1f + rootY, 0.5f -viewZ );
		
		/* Night Look */
		glColor4f( 0.0f, 1.0f, 0.0f, 0.3f);
		glVertex3d(-1.0f,-1.0f, 1.0f );
		glVertex3d( 1.0f,-1.0f, 1.0f );
		glVertex3d( 1.0f, 1.0f, 1.0f );
		glVertex3d(-1.0f, 1.0f, 1.0f );
	/* End Quads */
	glEnd();

	/* Disable */
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_BLEND);
}

/*-------------------------------------------------------------------------*/

/* Process Event */
void process_event(XEvent report)
{   
	KeySym key;
	key = XLookupKeysym (&report.xkey, 0);
		
	switch(key)
	{		
		case XK_Up:
		pitch -= 0.2f;
		break;
		
		case XK_Down:
		pitch += 0.2f;
		break;
		
		case XK_Left:
		roll += 0.5f;
		break;
		
		case XK_Right:
		roll -= 0.5f;
		break;
		
		case XK_a:
		planeX -= 0.05f;
		break;
		
		case XK_d:
		planeX += 0.05f;
		break;
		
		case XK_w:
		planeY += 0.04f;
		break;
		
		case XK_s:
		planeY -= 0.04f;
		break;
		
		case XK_e:
		yaw += 0.2f;
		break;
		
		case XK_q:
		yaw -= 0.2f;
		break;
		
		case XK_b:
		planeZ += 0.06f;
		break;
		
		case XK_space:
		planeZ -= 0.06f;
		break;
		
		case XK_m:
		viewZ = 0.0f;
		break;
		
		case XK_n:
		viewZ = -0.5f;
		break;
		
		case XK_F1:
		rootY = -0.6f;
		break;
		
		case XK_F2:
		rootY = +0.6f;
		break;
		
		case XK_F3:
		VA0 = +0.0f;
		VA1 = -0.1f;
		VA5 = +0.5f;
		VA10 = +1.0f;
		break;
		
		case XK_F4:
		VA0 = -0.0f;
		VA1 = +0.1f;    
		VA5 = -0.5f;
		VA10 = -1.0f;
		break;
		
		/* Close Window */
		case XK_Escape:
		closeWin();
		break;
	}
}

/*---------------------------------------------------------------------------*/


