/*********************************************************************
*                                                                    *          
*                              FLY                                   *
*                           --E#Y#E--				     *
*                         ===MUSCLE===                               *
*                                                                    *
*********************************************************************/

/* Header Files */
#include "uniform.h"	

/*-------------------------------------------------------------------------*/

#define MAX 2048

/*-------------------------------------------------------------------------*/

/* Create Shader */
void CreateShader()
{	
	char vert[MAX];
	char frag[MAX];

	/* Open Shader */
	v_vert=open("WIN/fly.vert",O_RDONLY);
	f_frag=open("WIN/fly.frag",O_RDONLY);

	/* Read File */
	read(v_vert, vert, MAX);
	read(f_frag, frag, MAX);

	/* Create Shader */
	shader.v.vertex = glCreateShader(GL_VERTEX_SHADER);
	shader.f.fragment = glCreateShader(GL_FRAGMENT_SHADER);

	/* Send to Source */
	glShaderSource(shader.v.vertex,vert[MAX],&shader.v.tmp,NULL);
	glShaderSource(shader.f.fragment,frag[MAX],&shader.f.tmp,NULL);
	
	/* Compile the Shaders */
	glCompileShader(shader.v.vertex);
	glGetShaderiv(shader.v.vertex, GL_COMPILE_STATUS, &shader.v.result);	
	if(shader.v.result)
	{
		printf("Could not Compile Shader. %d ", shader.v.vertex);
		
	}
	glGetShaderiv(shader.v.vertex, GL_INFO_LOG_LENGTH, &shader.v.info);
	if(shader.v.info)
	{	
		glGetShaderInfoLog(shader.v.vertex, vert[MAX], 
				&shader.v.info, &shader.v.log[0]);
		printf("Shader contains Errors! %s ", shader.v.log);	
	}

	/* Compile the Shaders */
	glCompileShader(shader.f.fragment);
	glGetShaderiv(shader.f.fragment, GL_COMPILE_STATUS, &shader.f.result);	
	if(shader.f.result)
	{
		printf("Could not Compile Shader. %d ", shader.f.fragment);
	}
	glGetShaderiv(shader.f.fragment, GL_INFO_LOG_LENGTH, &shader.f.info);
	if(shader.f.info)
	{	
		glGetShaderInfoLog(shader.f.fragment, frag[MAX], 
				&shader.f.info, &shader.f.log[0]);
		printf("Shader contains Errors! %s ", shader.f.log);
	}	
}

/*-------------------------------------------------------------------------*/

/* Create Program */	
void CreateProgram()
{	
	/* Create Shader */
	shader.u.program = glCreateProgram();

	/* Attach the Shaders to Object */
	glAttachShader(shader.u.program, shader.v.vertex);
	glAttachShader(shader.u.program, shader.f.fragment);
	
	/* Link the Program */	
	glLinkProgram(shader.u.program);

	/* Bind Program */
	glUseProgram(shader.u.program);
}

/*-------------------------------------------------------------------------*/

/* Location */
void Location(unsigned int index, const char* bind)
{
	/* Get Location */	
	location = glGetAttribLocation(shader.u.program, bind);
	glBindAttribLocation(shader.u.program, index, bind);
}	

/*-------------------------------------------------------------------------*/

/* Uniform */
void Uniform(const char* name ,const int id)
{
	/* Get Location */
	location = glGetUniformLocation(shader.u.program, name);
	glUniform1f(location, id);
}

/*-------------------------------------------------------------------------*/

/* Delete Shader */
void DeleteShader()
{
	/* Delete the Shaders */
	glDetachShader(shader.u.program, shader.v.vertex);	
	glDetachShader(shader.u.program, shader.f.fragment);
	glDeleteShader(shader.v.vertex);
	glDeleteShader(shader.f.fragment);
	glDeleteShader(shader.u.program);

	/* File Close */
	close(v_vert);
	close(f_frag);	
}

/*-------------------------------------------------------------------------*/

